@@ -3,7 +3,8 @@ import logging
 import socket
 import traceback
 import xml.etree.ElementTree as ET
-from typing import Callable, Optional, Type, Union
+from types import TracebackType
+from typing import Any, Callable, Dict, List, Optional, Type, Union
 
 from . import introspection as intr
 from ._private.address import get_bus_address, parse_address
@@ -20,10 +21,70 @@ from .constants import (
 from .errors import DBusError, InvalidAddressError
 from .message import Message
 from .proxy_object import BaseProxyObject
-from .service import ServiceInterface
+from .service import ServiceInterface, _Method
 from .signature import Variant
 from .validators import assert_bus_name_valid, assert_object_path_valid
 
+MESSAGE_TYPE_CALL = MessageType.METHOD_CALL
+MESSAGE_TYPE_SIGNAL = MessageType.SIGNAL
+NO_REPLY_EXPECTED_VALUE = MessageFlag.NO_REPLY_EXPECTED.value
+
+
+def _expects_reply(msg) -> bool:
+    return not (msg.flags.value & NO_REPLY_EXPECTED_VALUE)
+
+
+class SendReply:
+    """A context manager to send a reply to a message."""
+
+    __slots__ = ("_bus", "_msg")
+
+    def __init__(self, bus: "BaseMessageBus", msg: Message) -> None:
+        """Create a new reply context manager."""
+        self._bus = bus
+        self._msg = msg
+
+    def __enter__(self):
+        return self
+
+    def __call__(self, reply: Message) -> None:
+        if _expects_reply(self._msg):
+            self._bus.send(reply)
+
+    def _exit(
+        self,
+        exc_type: Optional[Type[Exception]],
+        exc_value: Optional[Exception],
+        tb: Optional[TracebackType],
+    ) -> bool:
+        if exc_type is None:
+            return False
+
+        if issubclass(exc_type, DBusError):
+            self(exc_value._as_message(self._msg))  # type: ignore[union-attr]
+            return True
+
+        if issubclass(exc_type, Exception):
+            self(
+                Message.new_error(
+                    self._msg,
+                    ErrorType.SERVICE_ERROR,
+                    f"The service interface raised an error: {exc_value}.\n{traceback.format_tb(tb)}",
+                )
+            )
+            return True
+
+    def __exit__(
+        self,
+        exc_type: Optional[Type[Exception]],
+        exc_value: Optional[Exception],
+        tb: Optional[TracebackType],
+    ) -> bool:
+        return self._exit(exc_type, exc_value, tb)
+
+    def send_error(self, exc: Exception) -> None:
+        self._exit(exc.__class__, exc, exc.__traceback__)
+
 
 class BaseMessageBus:
     """An abstract class to manage a connection to a DBus message bus.
@@ -55,29 +116,56 @@ class BaseMessageBus:
     :vartype connected: bool
     """
 
+    __slots__ = (
+        "unique_name",
+        "_disconnected",
+        "_user_disconnect",
+        "_method_return_handlers",
+        "_serial",
+        "_user_message_handlers",
+        "_name_owners",
+        "_path_exports",
+        "_bus_address",
+        "_name_owner_match_rule",
+        "_match_rules",
+        "_high_level_client_initialized",
+        "_ProxyObject",
+        "_machine_id",
+        "_negotiate_unix_fd",
+        "_sock",
+        "_stream",
+        "_fd",
+    )
+
     def __init__(
         self,
         bus_address: Optional[str] = None,
         bus_type: BusType = BusType.SESSION,
         ProxyObject: Optional[Type[BaseProxyObject]] = None,
-    ):
-        self.unique_name = None
+        negotiate_unix_fd: bool = False,
+    ) -> None:
+        self.unique_name: Optional[str] = None
         self._disconnected = False
+        self._negotiate_unix_fd = negotiate_unix_fd
 
         # True if the user disconnected himself, so don't throw errors out of
         # the main loop.
         self._user_disconnect = False
 
-        self._method_return_handlers = {}
+        self._method_return_handlers: Dict[
+            int, Callable[[Optional[Message], Optional[Exception]], None]
+        ] = {}
         self._serial = 0
-        self._user_message_handlers = []
+        self._user_message_handlers: List[
+            Callable[[Message], Union[Message, bool, None]]
+        ] = []
         # the key is the name and the value is the unique name of the owner.
         # This cache is kept up to date by the NameOwnerChanged signal and is
         # used to route messages to the correct proxy object. (used for the
         # high level client only)
-        self._name_owners = {}
+        self._name_owners: Dict[str, str] = {}
         # used for the high level service
-        self._path_exports = {}
+        self._path_exports: Dict[str, list[ServiceInterface]] = {}
         self._bus_address = (
             parse_address(bus_address)
             if bus_address
@@ -88,22 +176,22 @@ class BaseMessageBus:
         self._name_owner_match_rule = "sender='org.freedesktop.DBus',interface='org.freedesktop.DBus',path='/org/freedesktop/DBus',member='NameOwnerChanged'"
         # _match_rules: the keys are match rules and the values are ref counts
         # (used for the high level client only)
-        self._match_rules = {}
+        self._match_rules: Dict[str, int] = {}
         self._high_level_client_initialized = False
         self._ProxyObject = ProxyObject
 
         # machine id is lazy loaded
-        self._machine_id = None
+        self._machine_id: Optional[int] = None
 
         self._setup_socket()
 
     @property
-    def connected(self):
+    def connected(self) -> bool:
         if self.unique_name is None or self._disconnected or self._user_disconnect:
             return False
         return True
 
-    def export(self, path: str, interface: ServiceInterface):
+    def export(self, path: str, interface: ServiceInterface) -> None:
         """Export the service interface on this message bus to make it available
         to other clients.
 
@@ -131,12 +219,12 @@ class BaseMessageBus:
                 )
 
         self._path_exports[path].append(interface)
-        ServiceInterface._add_bus(interface, self)
+        ServiceInterface._add_bus(interface, self, self._make_method_handler)
         self._emit_interface_added(path, interface)
 
     def unexport(
         self, path: str, interface: Optional[Union[ServiceInterface, str]] = None
-    ):
+    ) -> None:
         """Unexport the path or service interface to make it no longer
         available to clients.
 
@@ -190,7 +278,7 @@ class BaseMessageBus:
         bus_name: str,
         path: str,
         callback: Callable[[Optional[intr.Node], Optional[Exception]], None],
-    ):
+    ) -> None:
         """Get introspection data for the node at the given path from the given
         bus name.
 
@@ -211,10 +299,10 @@ class BaseMessageBus:
         """
         BaseMessageBus._check_callback_type(callback)
 
-        def reply_notify(reply, err):
+        def reply_notify(reply: Optional[Message], err: Optional[Exception]) -> None:
             try:
                 BaseMessageBus._check_method_return(reply, err, "s")
-                result = intr.Node.parse(reply.body[0])
+                result = intr.Node.parse(reply.body[0])  # type: ignore[union-attr]
             except Exception as e:
                 callback(None, e)
                 return
@@ -231,7 +319,7 @@ class BaseMessageBus:
             reply_notify,
         )
 
-    def _emit_interface_added(self, path, interface):
+    def _emit_interface_added(self, path: str, interface: ServiceInterface) -> None:
         """Emit the ``org.freedesktop.DBus.ObjectManager.InterfacesAdded`` signal.
 
         This signal is intended to be used to alert clients when
@@ -246,7 +334,12 @@ class BaseMessageBus:
         if self._disconnected:
             return
 
-        def get_properties_callback(interface, result, user_data, e):
+        def get_properties_callback(
+            interface: ServiceInterface,
+            result: Any,
+            user_data: Any,
+            e: Optional[Exception],
+        ) -> None:
             if e is not None:
                 try:
                     raise e
@@ -272,7 +365,7 @@ class BaseMessageBus:
 
         ServiceInterface._get_all_property_values(interface, get_properties_callback)
 
-    def _emit_interface_removed(self, path, removed_interfaces):
+    def _emit_interface_removed(self, path: str, removed_interfaces: List[str]) -> None:
         """Emit the ``org.freedesktop.DBus.ObjectManager.InterfacesRemoved` signal.
 
         This signal is intended to be used to alert clients when
@@ -303,7 +396,7 @@ class BaseMessageBus:
         callback: Optional[
             Callable[[Optional[RequestNameReply], Optional[Exception]], None]
         ] = None,
-    ):
+    ) -> None:
         """Request that this message bus owns the given name.
 
         :param name: The name to request.
@@ -322,38 +415,41 @@ class BaseMessageBus:
         if callback is not None:
             BaseMessageBus._check_callback_type(callback)
 
-        def reply_notify(reply, err):
-            try:
-                BaseMessageBus._check_method_return(reply, err, "u")
-                result = RequestNameReply(reply.body[0])
-            except Exception as e:
-                callback(None, e)
-                return
-
-            callback(result, None)
-
         if type(flags) is not NameFlag:
             flags = NameFlag(flags)
 
-        self._call(
-            Message(
-                destination="org.freedesktop.DBus",
-                path="/org/freedesktop/DBus",
-                interface="org.freedesktop.DBus",
-                member="RequestName",
-                signature="su",
-                body=[name, flags],
-            ),
-            reply_notify if callback else None,
+        message = Message(
+            destination="org.freedesktop.DBus",
+            path="/org/freedesktop/DBus",
+            interface="org.freedesktop.DBus",
+            member="RequestName",
+            signature="su",
+            body=[name, flags],
         )
 
+        if callback is None:
+            self._call(message, None)
+            return
+
+        def reply_notify(reply: Optional[Message], err: Optional[Exception]) -> None:
+            try:
+                BaseMessageBus._check_method_return(reply, err, "u")
+                result = RequestNameReply(reply.body[0])  # type: ignore[union-attr]
+            except Exception as e:
+                callback(None, e)  # type: ignore[misc]
+                return
+
+            callback(result, None)  # type: ignore[misc]
+
+        self._call(message, reply_notify)
+
     def release_name(
         self,
         name: str,
         callback: Optional[
             Callable[[Optional[ReleaseNameReply], Optional[Exception]], None]
         ] = None,
-    ):
+    ) -> None:
         """Request that this message bus release the given name.
 
         :param name: The name to release.
@@ -371,27 +467,30 @@ class BaseMessageBus:
         if callback is not None:
             BaseMessageBus._check_callback_type(callback)
 
-        def reply_notify(reply, err):
+        message = Message(
+            destination="org.freedesktop.DBus",
+            path="/org/freedesktop/DBus",
+            interface="org.freedesktop.DBus",
+            member="ReleaseName",
+            signature="s",
+            body=[name],
+        )
+
+        if callback is None:
+            self._call(message, None)
+            return
+
+        def reply_notify(reply: Optional[Message], err: Optional[Exception]) -> None:
             try:
                 BaseMessageBus._check_method_return(reply, err, "u")
-                result = ReleaseNameReply(reply.body[0])
+                result = ReleaseNameReply(reply.body[0])  # type: ignore[union-attr]
             except Exception as e:
-                callback(None, e)
+                callback(None, e)  # type: ignore[misc]
                 return
 
-            callback(result, None)
+            callback(result, None)  # type: ignore[misc]
 
-        self._call(
-            Message(
-                destination="org.freedesktop.DBus",
-                path="/org/freedesktop/DBus",
-                interface="org.freedesktop.DBus",
-                member="ReleaseName",
-                signature="s",
-                body=[name],
-            ),
-            reply_notify if callback else None,
-        )
+        self._call(message, reply_notify)
 
     def get_proxy_object(
         self, bus_name: str, path: str, introspection: Union[intr.Node, str, ET.Element]
@@ -427,7 +526,7 @@ class BaseMessageBus:
 
         return self._ProxyObject(bus_name, path, introspection, self)
 
-    def disconnect(self):
+    def disconnect(self) -> None:
         """Disconnect the message bus by closing the underlying connection asynchronously.
 
         All pending  and future calls will error with a connection error.
@@ -451,7 +550,7 @@ class BaseMessageBus:
 
     def add_message_handler(
         self, handler: Callable[[Message], Optional[Union[Message, bool]]]
-    ):
+    ) -> None:
         """Add a custom message handler for incoming messages.
 
         The handler should be a callable that takes a :class:`Message
@@ -476,7 +575,7 @@ class BaseMessageBus:
 
     def remove_message_handler(
         self, handler: Callable[[Message], Optional[Union[Message, bool]]]
-    ):
+    ) -> None:
         """Remove a message handler that was previously added by
         :func:`add_message_handler()
         <dbus_fast.message_bus.BaseMessageBus.add_message_handler>`.
@@ -487,7 +586,7 @@ class BaseMessageBus:
         for i, h in enumerate(self._user_message_handlers):
             if h == handler:
                 del self._user_message_handlers[i]
-                break
+                return
 
     def send(self, msg: Message) -> None:
         """Asynchronously send a message on the message bus.
@@ -499,7 +598,7 @@ class BaseMessageBus:
             'the "send" method must be implemented in the inheriting class'
         )
 
-    def _finalize(self, err):
+    def _finalize(self, err: Optional[Exception]) -> None:
         """should be called after the socket disconnects with the disconnection
         error to clean up resources and put the bus in a disconnected state"""
         if self._disconnected:
@@ -531,8 +630,14 @@ class BaseMessageBus:
         return False
 
     def _interface_signal_notify(
-        self, interface, interface_name, member, signature, body, unix_fds=[]
-    ):
+        self,
+        interface: ServiceInterface,
+        interface_name: str,
+        member: str,
+        signature: str,
+        body: List[Any],
+        unix_fds: List[int] = [],
+    ) -> None:
         path = None
         for p, ifaces in self._path_exports.items():
             for i in ifaces:
@@ -555,7 +660,7 @@ class BaseMessageBus:
             )
         )
 
-    def _introspect_export_path(self, path):
+    def _introspect_export_path(self, path: str) -> intr.Node:
         assert_object_path_valid(path)
 
         if path in self._path_exports:
@@ -582,7 +687,7 @@ class BaseMessageBus:
 
         return node
 
-    def _setup_socket(self):
+    def _setup_socket(self) -> None:
         err = None
 
         for transport, options in self._bus_address:
@@ -598,7 +703,7 @@ class BaseMessageBus:
                 if "path" in options:
                     filename = options["path"]
                 elif "abstract" in options:
-                    filename = f'\0{options["abstract"]}'
+                    filename = b"\0" + options["abstract"].encode()
                 else:
                     raise InvalidAddressError(
                         "got unix transport with unknown path specifier"
@@ -634,19 +739,24 @@ class BaseMessageBus:
         if err:
             raise err
 
-    def _call(self, msg, callback, check_callback: bool = True) -> None:
+    def _call(
+        self,
+        msg: Message,
+        callback: Optional[Callable[[Optional[Message], Optional[Exception]], None]],
+        check_callback: bool = True,
+    ) -> None:
         if check_callback:
             BaseMessageBus._check_callback_type(callback)
 
         if not msg.serial:
             msg.serial = self.next_serial()
 
-        def reply_notify(reply, err):
-            if reply:
+        def reply_notify(reply: Optional[Message], err: Optional[Exception]) -> None:
+            if reply and msg.destination and reply.sender:
                 self._name_owners[msg.destination] = reply.sender
-            callback(reply, err)
+            callback(reply, err)  # type: ignore[misc]
 
-        no_reply_expected = msg.flags & MessageFlag.NO_REPLY_EXPECTED
+        no_reply_expected = not _expects_reply(msg)
 
         # Make sure the return reply handler is installed
         # before sending the message to avoid a race condition
@@ -658,10 +768,10 @@ class BaseMessageBus:
         self.send(msg)
 
         if no_reply_expected:
-            callback(None, None)
+            callback(None, None)  # type: ignore[misc]
 
     @staticmethod
-    def _check_callback_type(callback):
+    def _check_callback_type(callback: Callable) -> None:
         """Raise a TypeError if the user gives an invalid callback as a parameter"""
 
         text = "a callback must be callable with two parameters"
@@ -674,9 +784,15 @@ class BaseMessageBus:
             raise TypeError(text)
 
     @staticmethod
-    def _check_method_return(msg, err, signature):
+    def _check_method_return(
+        msg: Optional[Message], err: Optional[Exception], signature: str
+    ) -> None:
         if err:
             raise err
+        elif msg is None:
+            raise DBusError(
+                ErrorType.INTERNAL_ERROR, "invalid message type for method call", msg
+            )
         elif (
             msg.message_type == MessageType.METHOD_RETURN and msg.signature == signature
         ):
@@ -688,66 +804,18 @@ class BaseMessageBus:
                 ErrorType.INTERNAL_ERROR, "invalid message type for method call", msg
             )
 
-    def _on_message(self, msg):
-        try:
-            self._process_message(msg)
-        except Exception as e:
-            logging.error(
-                f"got unexpected error processing a message: {e}.\n{traceback.format_exc()}"
-            )
-
-    def _send_reply(self, msg):
-        bus = self
-
-        class SendReply:
-            def __enter__(self):
-                return self
-
-            def __call__(self, reply):
-                if msg.flags & MessageFlag.NO_REPLY_EXPECTED:
-                    return
-
-                bus.send(reply)
-
-            def _exit(self, exc_type, exc_value, tb):
-                if exc_type is None:
-                    return
-
-                if issubclass(exc_type, DBusError):
-                    self(exc_value._as_message(msg))
-                    return True
-
-                if issubclass(exc_type, Exception):
-                    self(
-                        Message.new_error(
-                            msg,
-                            ErrorType.SERVICE_ERROR,
-                            f"The service interface raised an error: {exc_value}.\n{traceback.format_tb(tb)}",
-                        )
-                    )
-                    return True
-
-            def __exit__(self, exc_type, exc_value, tb):
-                self._exit(exc_type, exc_value, tb)
-
-            def send_error(self, exc):
-                self._exit(exc.__class__, exc, exc.__traceback__)
-
-        return SendReply()
-
-    def _process_message(self, msg):
+    def _process_message(self, msg) -> None:
         handled = False
-
-        for handler in self._user_message_handlers:
+        for user_handler in self._user_message_handlers:
             try:
-                result = handler(msg)
+                result = user_handler(msg)
                 if result:
                     if type(result) is Message:
                         self.send(result)
                     handled = True
                     break
             except DBusError as e:
-                if msg.message_type == MessageType.METHOD_CALL:
+                if msg.message_type is MESSAGE_TYPE_CALL:
                     self.send(e._as_message(msg))
                     handled = True
                     break
@@ -759,7 +827,7 @@ class BaseMessageBus:
                 logging.error(
                     f"A message handler raised an exception: {e}.\n{traceback.format_exc()}"
                 )
-                if msg.message_type == MessageType.METHOD_CALL:
+                if msg.message_type is MESSAGE_TYPE_CALL:
                     self.send(
                         Message.new_error(
                             msg,
@@ -770,24 +838,25 @@ class BaseMessageBus:
                     handled = True
                     break
 
-        if msg.message_type == MessageType.SIGNAL:
-            if msg._matches(
-                member="NameOwnerChanged",  # least likely to match
-                sender="org.freedesktop.DBus",
-                path="/org/freedesktop/DBus",
-                interface="org.freedesktop.DBus",
+        if msg.message_type is MESSAGE_TYPE_SIGNAL:
+            if (
+                msg.member == "NameOwnerChanged"
+                and msg.sender == "org.freedesktop.DBus"
+                and msg.path == "/org/freedesktop/DBus"
+                and msg.interface == "org.freedesktop.DBus"
             ):
                 [name, old_owner, new_owner] = msg.body
                 if new_owner:
                     self._name_owners[name] = new_owner
                 elif name in self._name_owners:
                     del self._name_owners[name]
+            return
 
-        elif msg.message_type == MessageType.METHOD_CALL:
+        if msg.message_type is MESSAGE_TYPE_CALL:
             if not handled:
                 handler = self._find_message_handler(msg)
 
-                send_reply = self._send_reply(msg)
+                send_reply = SendReply(self, msg)
 
                 with send_reply:
                     if handler:
@@ -800,74 +869,96 @@ class BaseMessageBus:
                                 f'{msg.interface}.{msg.member} with signature "{msg.signature}" could not be found',
                             )
                         )
+            return
 
-        else:
-            # An ERROR or a METHOD_RETURN
-            if msg.reply_serial in self._method_return_handlers:
-                if not handled:
-                    handler = self._method_return_handlers[msg.reply_serial]
-                    handler(msg, None)
-                del self._method_return_handlers[msg.reply_serial]
+        # An ERROR or a METHOD_RETURN
+        if msg.reply_serial in self._method_return_handlers:
+            if not handled:
+                return_handler = self._method_return_handlers[msg.reply_serial]
+                return_handler(msg, None)
+            del self._method_return_handlers[msg.reply_serial]
 
-    def _make_method_handler(self, interface, method):
-        def handler(msg, send_reply):
+    def _make_method_handler(
+        self, interface: ServiceInterface, method: _Method
+    ) -> Callable[[Message, Callable[[Message], None]], None]:
+        def handler(msg: Message, send_reply: Callable[[Message], None]) -> None:
             args = ServiceInterface._msg_body_to_args(msg)
             result = method.fn(interface, *args)
             body, fds = ServiceInterface._fn_result_to_body(
-                result, signature_tree=method.out_signature_tree
+                result,
+                signature_tree=method.out_signature_tree,
+                replace_fds=self._negotiate_unix_fd,
+            )
+            send_reply(
+                Message(
+                    message_type=MessageType.METHOD_RETURN,
+                    reply_serial=msg.serial,
+                    destination=msg.sender,
+                    signature=method.out_signature,
+                    body=body,
+                    unix_fds=fds,
+                )
             )
-            send_reply(Message.new_method_return(msg, method.out_signature, body, fds))
 
         return handler
 
-    def _find_message_handler(self, msg):
-        handler = None
+    def _find_message_handler(
+        self, msg
+    ) -> Optional[Callable[[Message, Callable], None]]:
+        handler: Optional[Callable[[Message, Callable], None]] = None
 
-        if msg._matches(
-            interface="org.freedesktop.DBus.Introspectable",
-            member="Introspect",
-            signature="",
+        if (
+            msg.interface == "org.freedesktop.DBus.Introspectable"
+            and msg.member == "Introspect"
+            and msg.signature == ""
         ):
             handler = self._default_introspect_handler
 
-        elif msg._matches(interface="org.freedesktop.DBus.Properties"):
+        elif msg.interface == "org.freedesktop.DBus.Properties":
             handler = self._default_properties_handler
 
-        elif msg._matches(interface="org.freedesktop.DBus.Peer"):
-            if msg._matches(member="Ping", signature=""):
+        elif msg.interface == "org.freedesktop.DBus.Peer":
+            if msg.member == "Ping" and msg.signature == "":
                 handler = self._default_ping_handler
-            elif msg._matches(member="GetMachineId", signature=""):
+            elif msg.member == "GetMachineId" and msg.signature == "":
                 handler = self._default_get_machine_id_handler
-        elif msg._matches(
-            interface="org.freedesktop.DBus.ObjectManager", member="GetManagedObjects"
+        elif (
+            msg.interface == "org.freedesktop.DBus.ObjectManager"
+            and msg.member == "GetManagedObjects"
         ):
             handler = self._default_get_managed_objects_handler
 
-        else:
+        elif msg.path:
             for interface in self._path_exports.get(msg.path, []):
                 for method in ServiceInterface._get_methods(interface):
                     if method.disabled:
                         continue
-                    if msg._matches(
-                        interface=interface.name,
-                        member=method.name,
-                        signature=method.in_signature,
+                    if (
+                        msg.interface == interface.name
+                        and msg.member == method.name
+                        and msg.signature == method.in_signature
                     ):
-                        handler = self._make_method_handler(interface, method)
+                        handler = ServiceInterface._get_handler(interface, method, self)
                         break
                 if handler:
                     break
 
         return handler
 
-    def _default_introspect_handler(self, msg, send_reply):
+    def _default_introspect_handler(
+        self, msg: Message, send_reply: Callable[[Message], None]
+    ) -> None:
         introspection = self._introspect_export_path(msg.path).tostring()
         send_reply(Message.new_method_return(msg, "s", [introspection]))
 
-    def _default_ping_handler(self, msg, send_reply):
+    def _default_ping_handler(
+        self, msg: Message, send_reply: Callable[[Message], None]
+    ) -> None:
         send_reply(Message.new_method_return(msg))
 
-    def _default_get_machine_id_handler(self, msg, send_reply):
+    def _default_get_machine_id_handler(
+        self, msg: Message, send_reply: Callable[[Message], None]
+    ) -> None:
         if self._machine_id:
             send_reply(Message.new_method_return(msg, "s", self._machine_id))
             return
@@ -898,7 +989,9 @@ class BaseMessageBus:
             check_callback=False,
         )
 
-    def _default_get_managed_objects_handler(self, msg, send_reply):
+    def _default_get_managed_objects_handler(
+        self, msg: Message, send_reply: Callable[[Message], None]
+    ) -> None:
         result = {}
         result_signature = "a{oa{sa{sv}}}"
         error_handled = False
@@ -948,7 +1041,9 @@ class BaseMessageBus:
                     interface, get_all_properties_callback, node
                 )
 
-    def _default_properties_handler(self, msg, send_reply):
+    def _default_properties_handler(
+        self, msg: Message, send_reply: Callable[[Message], None]
+    ) -> None:
         methods = {"Get": "ss", "Set": "ssv", "GetAll": "s"}
         if msg.member not in methods or methods[msg.member] != msg.signature:
             raise DBusError(
@@ -1090,7 +1185,7 @@ class BaseMessageBus:
         else:
             assert False
 
-    def _init_high_level_client(self):
+    def _init_high_level_client(self) -> None:
         """The high level client is initialized when the first proxy object is
         gotten. Currently just sets up the match rules for the name owner cache
         so signals can be routed to the right objects."""
@@ -1134,7 +1229,7 @@ class BaseMessageBus:
 
         self._match_rules[match_rule] = 1
 
-        def add_match_notify(msg, err):
+        def add_match_notify(msg: Message, err: Optional[Exception]) -> None:
             if err:
                 logging.error(f'add match request failed. match="{match_rule}", {err}')
             if msg.message_type == MessageType.ERROR:
